﻿using Infragistics.Windows.Ribbon;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Reflection;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace XamRibbon_BindingGallery
{
    public class RibbonViewModel : INotifyPropertyChanged
    {
        public GalleryItem selected { get; set; }

        private ObservableCollection<GalleryItem> _galleryItems;
        public ObservableCollection<GalleryItem> GalleryItems
        {
            get { return _galleryItems; }
            set
            {
                _galleryItems = value;
                NotifyPropertyChanged("GalleryItems");
            }
        }

        public RibbonViewModel()
        {
            GalleryItems = GenerateColorGalleryItems();
        }

        /// <summary>
        /// Generates a collection of GalleryItems with all available colors.
        /// </summary>
        ObservableCollection<GalleryItem> GenerateColorGalleryItems()
        {
            ObservableCollection<GalleryItem> items = new ObservableCollection<GalleryItem>();

            PropertyInfo[] properties = typeof(Colors).GetProperties(BindingFlags.Static | BindingFlags.Public);
            foreach (PropertyInfo info in properties)
            {
                Color c = (Color)info.GetValue(typeof(Colors), null);
                ImageSource src = CreateImage(c, new Size(60, 20));
                items.Add(CreateItem(info.Name, src, c));
            }

            return items;
        }

        /// <summary>
        /// Creates a color gallery item item.
        /// </summary>
        /// <param name="caption">The caption.</param>
        /// <param name="image">The image.</param>
        /// <param name="color">The color.</param>
        /// <returns></returns>
        static GalleryItem CreateItem(string caption, ImageSource image, Color color)
        {
            GalleryItem item = new GalleryItem { Key = string.Format("Color_{0}", caption), Image = image, Text = caption, Tag = color };
            return item;
        }

        /// <summary>
        /// Creates the image for a color gallery item
        /// </summary>
        /// <param name="color">The color.</param>
        /// <param name="size">The size.</param>
        /// <returns></returns>
        static ImageSource CreateImage(Color color, Size size)
        {
            DrawingVisual v = new DrawingVisual();
            DrawingContext c = v.RenderOpen();
            c.DrawRectangle(new SolidColorBrush(color), null, new Rect(0, 0, size.Width, size.Height));
            c.Close();

            RenderTargetBitmap rtb = new RenderTargetBitmap((int)size.Width, (int)size.Height, 96, 96, PixelFormats.Pbgra32);
            rtb.Render(v);
            return rtb;
        }


        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }

    public class GalleryTool
    {
        public static readonly DependencyProperty ItemsProperty =
            DependencyProperty.RegisterAttached("Items", typeof(ObservableCollection<GalleryItem>), typeof(GalleryTool), new UIPropertyMetadata(null, new PropertyChangedCallback(OnItemsPropertyChanged)));

        public static ObservableCollection<GalleryItem> GetItems(DependencyObject obj)
        {
            return (ObservableCollection<GalleryItem>)obj.GetValue(ItemsProperty);
        }

        public static void SetItems(DependencyObject obj, ObservableCollection<GalleryItem> value)
        {
            obj.SetValue(ItemsProperty, value);
        }

        static void OnItemsPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            Infragistics.Windows.Ribbon.GalleryTool tool = obj as Infragistics.Windows.Ribbon.GalleryTool;
            if (tool == null)
                throw new ArgumentException("Items can only be set on a GalleryTool.");

            ObservableCollection<GalleryItem> items = e.NewValue as ObservableCollection<GalleryItem>;
            if (items != null)
            {
                tool.Items.AddRange(items);
            }
        }
    }
}